package	jp.co.sds_corp.fixconnect.logger;

import	java.io.BufferedOutputStream;
import	java.io.File;
import	java.io.FileOutputStream;
import	java.io.IOException;
import	java.io.OutputStream;
import	java.io.OutputStreamWriter;
import	java.io.Writer;

import	jp.co.sds_corp.fixconnect.FIXException;
import	jp.co.sds_corp.fixconnect.message.FIXMessage;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class FileLogger extends AbstractMessageLogger
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	String	dir;
	String	filename;
	String	path;
	OutputStream	os = null;
	Writer	wr = null;

	public void init( IConfiguration config ) throws Exception {
		super.init( config );

		IConfiguration attr = config.getAttributes();
		dir = attr.get( "dir" );
		filename = attr.get( "filename" );

		path = dir + File.separator + filename;
		File file = new File( path );
		try {
			os = new BufferedOutputStream( new FileOutputStream( file, true ));
			wr = new OutputStreamWriter( os );
		} catch ( Exception ex ) {
			throw( new FIXException( "logfile[" + path + "] open error" ));
		}
		logger.debug( this, "open log file: " + path );
	}

	public void log( Direction direction, FIXMessage msg ) throws FIXException {
		if ( !isLogging( msg )) {
			return;
		}

		// logger.debug( this, direction + " file: " + path );
		try {
            os.write( msg.getBuffer() );
            if ( isDump()) {
                wr.write( "\n" + dump( msg ));
				wr.flush();
            }
            os.flush();
		} catch ( Exception ex ) {
			throw( new FIXException( "log write error", ex ));
        }
	}

	public void close() throws FIXException {
    	try {
			wr.flush();
			wr.close();
        } catch ( IOException ex ) {
			throw( new FIXException( "logfile close error", ex ));
        }
	}

}
