package	jp.co.sds_corp.fixconnect.message;

import	java.io.OutputStream;
import	java.util.HashMap;
import	java.util.List;
import	java.util.Map;
import	java.util.Vector;

import jp.co.sds_corp.fixconnect.FIXException;

public class Component extends Field
{
	/** serialVersionUID */
    private static final long serialVersionUID = 1L;

	static class Group
	{
		List<Field> listField = new Vector<Field>();
		Map<String,Field> mapField = new HashMap<String,Field>();
		Group() {
		}
		Group( Group org ) {
			for ( int i = 0; i < org.listField.size(); i++ ) {
				Field orgFld = org.listField.get( i );
				Field newFld = orgFld.cloneField();
				listField.add( newFld );
				mapField.put( newFld.getName(), newFld );
			}
		}
	}

	List<Group> listGroup = new Vector<Group>();
	int	count;
	Field	countField;
	String	name;
	boolean	repeat;
	boolean	showNull = false;

	public Component( String name ) {
		this( name, 1 );
	}

	public Component( String name, int count ) {
		this.name = name;
		this.count = count;
		for ( int i = 0; i < count; i++ ) {
			Group group = new Group();
			listGroup.add( group );
		}
	}

	protected Component( Component org ) {
		this.name = org.name;
		this.repeat = org.repeat;
		this.countField = org.countField;
		this.count = org.count;
		for ( int i = 0; i < count; i++ ) {
			Group orgGrp = org.listGroup.get( i );
			Group group = new Group( orgGrp );
			listGroup.add( group );
		}
	}

	public Field cloneField() {
		Component ret = new Component( this );
		return ret;
	}

	public void addField( Field field ) {
		for ( int i = 0; i < listGroup.size(); i++ ) {
			Field newFld;
			if ( i == 0 ) {
				newFld = field;
			} else {
				newFld = field.cloneField();
			}
			Group group = listGroup.get( i );
			group.mapField.put( field.getName(), newFld );
			group.listField.add( newFld );
		}
	}

	/**
	* 電文項目を取得します.
	* @param	id		タグID
	* @param	index	コンポーネントのindex
	* @return	項目
	* @throws	FieldNotFoundException
	*/
	public Field findFieldById( int id, int index )
		throws FieldNotFoundException
	{
		Field field;
		field = getCountField();
		if ( field != null && field.getId() == id ) {
			return field;
		}
		List<Field> list = listField( index );
		for ( int i = 0; i < list.size(); i++ ) {
			field = list.get( i );
			if ( field instanceof Component ) {
				Component c = (Component )field;
				Field ret = c.findFieldById( id, 0 );
				if ( ret != null ) {
					return ret;
				}
			} else if ( field.getId() == id ) {
				return field;
			}
		}
		return null;
	}

	protected Field findField( String name ) throws Exception {
		return findField( name, 0 );
	}

	protected Field findField( String name, int index ) {
		if ( index > ( listGroup.size() - 1 )) {
			setCount( index + 1 );
		}
		Group group = listGroup.get( index );
		Field ret = group.mapField.get( name );
		return ret;
	}

	/**
	* フィールドの一覧を返します。
	* @return   フィールドの一覧
	*/
	public List<Field> listField() {
		return listField( 0 );
	}

	/**
	* フィールドの一覧を返します。
	* @return   フィールドの一覧
	*/
	public List<Field> listField( int index ) {
		Group group = listGroup.get( index );
		return group.listField;
	}

	/**
	* コンポーネント名を返します。
	*/
	public String getName() {
		return this.name;
	}

	public void setCount( int count ) {
		if ( this.count < count ) {
			Group org = listGroup.get( 0 );
			for ( int i = this.count; i < count; i++ ) {
				Group group = new Group( org );
				listGroup.add( group );
			}
/*
		} else if ( this.count > count ) {
			listGroup.setSize( count );
			throw( new IDealSysException( "count less than " + this.count + "." ));
*/
		}
		this.count = count;
	}

	public int getCount() {
		return count;
	}

	public void setCountField( Field countField ) {
		this.countField = countField;
	}

	public Field getCountField() {
		return countField;
	}

	protected void setRepeat( boolean repeat ) {
		this.repeat = repeat;
	}

	public boolean isRepeat() {
		return this.repeat;
	}

	private int getRealCount() {
		if ( !isRepeat()) {
			return 1;
		}
		int cnt = count;
		if ( repeat ) {
			try {
				cnt = Integer.valueOf( countField.getValue() ).intValue();
			} catch ( Exception ex ) {
				cnt = 0;
			}
		}
		return cnt;
	}

	public String toString() {
		return toString( 0 );
	}

	public String toString( int depth ) {
		String ret = "";
		int cnt = getRealCount();
		if ( cnt == 0 ) {
			for ( int k = 0; k < depth; k++ ) {
				ret += "\t";
			}
			ret += "- " + name + ": " + countField.toString();
		}
		if ( cnt > listGroup.size() ) {
			throw( new ArrayIndexOutOfBoundsException(
				"Array index out of range: " + cnt
				+ " " + name + "=" + listGroup.size() ));
		}
		for ( int i = 0; i < cnt; i++ ) {
			if ( i > 0 ) {
				ret += "\n";
			}
			for ( int k = 0; k < depth; k++ ) {
				ret += "\t";
			}
			if ( count > 1 ) {
				ret += "+ " + name + "[" + i + "]:";
			} else {
				ret += "+ " + name + ":";
			}
			if ( countField != null ) {
				ret += " " + countField.toString();
			}
			ret += "\n";
			Group group = listGroup.get( i );
			for ( int j = 0; j < group.listField.size(); j++ ) {
				Field field = group.listField.get( j );
				if ( !showNull ) {
					if ( field instanceof Component ) {
						if ( ((Component )field).getRealCount() == 0 ) {
							continue;
						}
					} else if ( field.getValue() == null ) {
						continue;
					}
				}
				if ( j > 0 ) {
					ret += "\n";
				}
				ret += field.toString( depth + 1 );
			}
		}
		return ret;
	}

	protected void serialize( OutputStream out ) throws FIXException {
		if ( listGroup.size() > count ) {
			throw( new FIXException( "count over."
				+ "(" + listGroup.size() + ")"
				+ " component=" + getName() ));
		}
		if ( countField != null && countField.getValue() != null ) {
			countField.serialize( out );
		}
		int cnt = getRealCount();
		for ( int i = 0; i < cnt; i++ ) {
			Group group = listGroup.get( i );
			for ( int j = 0; j < group.listField.size(); j++ ) {
				Field field = group.listField.get( j );
				if ( field.getName().equals( Tag.BeginString.getName()) ||
					field.getName().equals( Tag.BodyLength.getName()) ||
					field.getName().equals( Tag.CheckSum.getName()))
				{
					continue;
				}
				field.serialize( out );
			}
		}
	}

}
