package	jp.co.sds_corp.fixconnect.message;

import	java.io.OutputStream;
import	java.io.Serializable;

import jp.co.sds_corp.fixconnect.FIXException;

/**
* 電文の一項目の定義情報とデータを保持します。
* 
*/
public class Field implements Serializable
{
	/** serialVersionUID */
    private static final long serialVersionUID = 1L;

	Tag	tag;
	int	length;	// 項目長
	String	value;	// 格納データ
	String	defaultValue;	// デフォルト値
	boolean	required;	// 必須フラグ
	Component	component;	// 属するComponent
	boolean	countField;	// グループ数であるか

	protected Field() {
	}

	/**
	* コピーコンストラクタ
	* @param	org		コピー元
	*/
	protected Field( Field org ) {
		this.tag = org.tag;
		this.length = org.length;
		this.defaultValue = org.defaultValue;
		this.required = org.required;
	}

	/**
	* 指定された情報で、項目を初期化します。
	* @param	tag		タグ
	*/
	public Field( Tag tag ) {
		this.tag = tag;
	}

	/**
	* 指定された情報で、項目を初期化します。
	* @param	id		TagId
	* @param	name	項目名
	*/
	public Field( int id, String name ) {
		this.tag = new Tag( id, name );
	}

	/**
	* 自分自身のコピーを返します。
	* @return	自分自身のコピー
	*/
	public Field cloneField() {
		Field ret = new Field( this );
		return ret;
	}

	/**
	* この項目の文字列表現を返します。
	* @return	この項目の文字列表現
	*/
	public String toString() {
		return toString( 0 );
	}

	public String toString( int depth ) {
		String ret = "";
		for ( int i = 0; i < depth; i++ ) {
			ret += "\t";
		}
		ret += getTag() + "=[" + getValue() + "]";
		return ret;
	}

	/**
	* この項目をバイト配列に変換して出力します。
	* @param    out     出力先
	*/
	protected void serialize( OutputStream out ) throws FIXException {
		try {
			if ( getValue() == null ) {
				return;
			}
			String var = getId() + "=" + getValue();
			out.write( var.getBytes() );
			out.write( FIXMessage.SOH );
		} catch ( Exception ex ) {
			throw( new FIXException( "serialize error", ex ));
		}
	}

	/**
	* 項目名を取得します。
	* @return	項目名
	*/
	public String getName() {
		return tag.getName();
	}

	/**
	* TagIdを取得します。
	* @return	TagId
	*/
	public int getId() {
		return tag.getId();
	}

	/**
	* Tagを取得します。
	* @return	Tag
	*/
	public Tag getTag() {
		return tag;
	}

	/**
	* 項目長を設定します。
	* @param	var		設定するデータ
	*/
	protected void setLength( int var ) {
		this.length = var;
	}
	/**
	* 項目長を取得します。
	* @return	項目長
	*/
	public int getLength() {
		return length;
	}

	/**
	* 格納データを設定します。
	* @param	var		設定するデータ
	*/
	public void setValue( String var ) {
		this.value = var;
	}
	/**
	* 格納データを取得します。
	* @return	格納データ
	*/
	public String getValue() {
		if ( value == null ) {
			return getDefaultValue();
		}
		return value;
	}

	/**
	* デフォルト値を設定します。
	* @param	var		設定するデータ
	*/
	protected void setDefaultValue( String var ) {
		this.defaultValue = var;
	}
	/**
	* デフォルト値を取得します。
	* @return	デフォルト値
	*/
	public String getDefaultValue() {
		return defaultValue;
	}

	/**
	* 必須フラグを設定します。
	* @param	var		設定するデータ
	*/
	protected void setRequired( boolean var ) {
		this.required = var;
	}
	/**
	* 必須フラグを取得します。
	* @return	必須フラグ
	*/
	public boolean isRequired() {
		return required;
	}

	/**
	* 属するComponentを設定します。
	* @param	var		設定するデータ
	*/
	protected void setComponent( Component var ) {
		this.component = var;
	}
	/**
	* 属するComponentを取得します。
	* @return	属するComponent
	*/
	public Component getComponent() {
		return component;
	}

	/**
	* グループ数であるかを設定します。
	* @param	var		設定するデータ
	*/
	protected void setCountField( boolean var ) {
		this.countField = var;
	}
	/**
	* グループ数であるかを取得します。
	* @return	グループ数であるか
	*/
	public boolean isCountField() {
		return countField;
	}



}
