package	jp.co.sds_corp.fixconnect.message;

import	java.io.Serializable;
import	java.util.List;
import	java.util.regex.Pattern;
import	java.util.regex.Matcher;

public class Layout implements Serializable
{
	/** serialVersionUID */
    private static final long serialVersionUID = 1L;

	private Component root = new Component( "" );

	protected Component getRootComponent() {
		return root;
	}

	protected void addField( Field field ) {
		root.addField( field );
	}

	/**
	* 電文項目を取得します.
	* @param	component	コンポーネント
	* @param	id		タグID
	* @return	項目
	* @throws	FieldNotFoundException
	*/
	public Field findFieldById( int id )
		throws FieldNotFoundException
	{
		return root.findFieldById( id, 0 );
	}

	/**
	* 電文項目を取得します.
	* @param	tag		項目名
	* @return	項目
	* @throws	FieldNotFoundException
	*/
	public Field findField( Tag tag )
		throws FieldNotFoundException
	{
		return findField( root, tag.getName() );
	}

	/**
	* 電文項目を取得します.
	* @param	name	項目名
	* @return	項目
	* @throws	FieldNotFoundException
	*/
	public Field findField( String name )
		throws FieldNotFoundException
	{
		return findField( root, name );
	}

	private Pattern pattern = Pattern.compile( "(.*)\\[(.*)\\]" );

	/**
	* 電文項目を取得します.
	* @param	component	コンポーネント
	* @param	name	項目名
	* @return	項目
	* @throws	FieldNotFoundException
	*/
	public Field findField( Component component, String name )
		throws FieldNotFoundException
	{
		return findField( component, 0, name );
	}

	/**
	* 電文項目を取得します.
	* @param	compName	コンポーネント名
	* @param	index	コンポーネントのindex
	* @param	name	項目名
	* @return	項目
	* @throws	FieldNotFoundException
	*/
	public Field findField( String compName, int index, String name )
		throws FieldNotFoundException
	{
		Component component = (Component )findField( compName );
		return findField( component, index, name );
	}

	/**
	* 電文項目を取得します.
	* @param	component	コンポーネント
	* @param	index	コンポーネントのindex
	* @return	項目
	* @throws	FieldNotFoundException
	*/
	public Field findField( Component component, int index, String name )
		throws FieldNotFoundException
	{
		Field ret = null;
		int idx = name.indexOf( "." );
		if ( idx < 0 ) {
			ret = component.findField( name, index );
			if ( ret == null ) {
				ret = component.getCountField();
				if ( !ret.getName().equals( name )) {
					String msg = "field[" + name + "] not found.";
					if ( component.getName().length() > 0 ) {
						msg += " in component=[" + component.getName() + "]";
					}
					throw( new FieldNotFoundException( msg ));
				}
			}
		} else {
			int cidx = 0;
			Component c = null;
			String cn = name.substring( 0, idx );
			String fn = name.substring( idx + 1 );
			Matcher matcher = pattern.matcher( cn );
			if ( matcher.find()) {
				cn = matcher.group( 1 );
				String sidx = matcher.group( 2 );
				try {
					cidx = Integer.valueOf( sidx ).intValue();
				} catch ( Exception ex ) {
					throw( new FieldNotFoundException( "illegal index"
							+ " component=[" + cn + "]"
							+ " index=[" + sidx + "]" ));
				}
			}
			Field field = component.findField( cn, index );
			if ( field == null ) {
				String msg = "component[" + cn + "] not found.";
				if ( component.getName().length() > 0 ) {
					msg += " in component=[" + component.getName() + "]";
				}
				throw( new FieldNotFoundException( msg ));
			}
			if ( ! ( field instanceof Component )) {
				throw( new FieldNotFoundException( "field[" + cn + "]"
							+ " not component." ));
			}
			c = (Component )field;
			ret = findField( c, cidx, fn );
		}
		return ret;
	}

	/**
	* フィールドの一覧を返します。
	* @return	フィールドの一覧
	*/
	public List<Field> listField() {
		return root.listField();
	}

}
