package	jp.co.sds_corp.fixconnect.transport;

import	java.util.Date;
import	java.util.Timer;
import	java.util.TimerTask;

import	jp.co.sds_corp.fixconnect.FIXException;
import	jp.co.sds_corp.fixconnect.message.FIXMessage;
import	jp.co.sds_corp.fixconnect.message.IMessageReceiver;
import	jp.co.sds_corp.fixconnect.message.MessageReceiveEvent;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* セッションの状態が変化したことを表すイベントです。
*/
public class HeartBeatChecker implements IMessageReceiver
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	TransportSupport transportSupport = new TransportSupport();
	FIXMessage	hbMsg;
	FIXMessage	testMsg;
	Timer	timer;
	CheckTask	checkTask;

	public HeartBeatChecker( FIXSession session ) throws FIXException {
		this.session = session;
		hbMsg = session.getParser().createMessage( "0" );
		testMsg = session.getParser().createMessage( "1" );
	}

	class CheckTask extends TimerTask {
		public void run() {
			FIXSession session = getSession();
			Date now = new Date();
			if ( getLastSent() == null ) {
				resetLastSent();
			}
			if (( now.getTime() - getLastSent().getTime()) >
					( session.getHeartBeat() * 1000 ))
			{
				if ( session.getState() != SessionState.LOGONED ) {
					return;
				}
				try {
					session.send( hbMsg );
				} catch ( FIXException ex ) {
					logger.error( this, "HeartBeat send error " + session, ex );
				}
				resetLastSent();
			}
			if ( getLastReceived() == null ) {
				resetLastReceived();
			}
			if (( now.getTime() - getLastReceived().getTime()) >
				(( session.getHeartBeat() + session.getHeartBeatTimeout() )
					* 1000 ))
			{
				HeartBeatTimeoutException tex =
					new HeartBeatTimeoutException( session );
				fireExceptionOccurred( tex );
				try {
					session.send( testMsg );
				} catch ( FIXException ex ) {
					logger.error( this, "TestRequest send error", ex );
				}
				// @@@
				resetLastReceived();
			}
		}

	}

	public void start() {
		logger.trace( this, "start HeartBeat" );
		timer = new Timer( "aidw.fix.HeartBeatChecker" );
		checkTask = new CheckTask();
		timer.schedule( checkTask, 0, 1000 );
	}

	public void stop() {
		logger.trace( this, "stop HeartBeat" );
		if ( timer != null ) {
			timer.cancel();
		}
	}

	public void onMessage( MessageReceiveEvent ev ) {
		FIXMessage msg = ev.getMessage();
		String type = msg.getMsgType();
		if ( type.equals( "0" )) {
		} else
		if ( type.equals( "1" )) {
			try {
				session.send( hbMsg );
			} catch ( FIXException ex ) {
				logger.error( this, "heartbeat response send error", ex );
			}
		} else {
		}
		resetLastReceived();
	}

	public synchronized void resetLastSent() {
		lastSent = new Date();
	}

	public synchronized void resetLastReceived() {
		lastReceived = new Date();
	}

	public void addEventListener( ITransportEventListener l ) {
		transportSupport.addEventListener( l );
	}
	public boolean removeEventListener( ITransportEventListener l ) {
		return transportSupport.removeEventListener( l );
	}
	public void fireEventOccurred( TransportEvent ev ) {
		transportSupport.fireEventOccurred( getSession(), ev );
	}
	public void fireExceptionOccurred( FIXException ex ) {
		transportSupport.fireExceptionOccurred( ex );
	}

	FIXSession	session;	// FIXセッション情報
	boolean	stop;	// 停止フラグ
	int	interval;	// チェック間隔
	Date	lastSent;	// 最終送信時刻
	Date	lastReceived;	// 最終受信時刻

	/**
	* FIXセッション情報を設定します。
	* @param	var		設定するデータ
	*/
	public void setSession( FIXSession var ) {
		this.session = var;
	}
	/**
	* FIXセッション情報を取得します。
	* @return	FIXセッション情報
	*/
	public synchronized FIXSession getSession() {
		return session;
	}

	/**
	* 停止フラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setStop( boolean var ) {
		this.stop = var;
	}
	/**
	* 停止フラグを取得します。
	* @return	停止フラグ
	*/
	public boolean isStop() {
		return stop;
	}

	/**
	* チェック間隔を設定します。
	* @param	var		設定するデータ
	*/
	public void setInterval( int var ) {
		this.interval = var;
	}
	/**
	* チェック間隔を取得します。
	* @return	チェック間隔
	*/
	public int getInterval() {
		return interval;
	}

	/**
	* 最終送信時刻を設定します。
	* @param	var		設定するデータ
	*/
	synchronized void setLastSent( Date var ) {
		this.lastSent = var;
	}
	/**
	* 最終送信時刻を取得します。
	* @return	最終送信時刻
	*/
	public synchronized Date getLastSent() {
		return lastSent;
	}

	/**
	* 最終受信時刻を設定します。
	* @param	var		設定するデータ
	*/
	synchronized void setLastReceived( Date var ) {
		this.lastReceived = var;
	}
	/**
	* 最終受信時刻を取得します。
	* @return	最終受信時刻
	*/
	public synchronized Date getLastReceived() {
		return lastReceived;
	}

}
