package	jp.co.sds_corp.fixconnect.test;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import jp.co.sds_corp.fixconnect.message.FIXMessage;
import jp.co.sds_corp.fixconnect.message.IMessageReceiver;
import jp.co.sds_corp.fixconnect.message.MessageReceiveEvent;
import jp.co.sds_corp.fixconnect.transport.FIXSession;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class DealServer implements IMessageReceiver
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	boolean stop;
	FIXSession session;
	Tenor	tenor = new Tenor();
	int	dealId = 1;
	DecimalFormat df = new DecimalFormat( "00000000" );
	SimpleDateFormat sdf1 = new SimpleDateFormat( "yyyyMMdd" );
	SimpleDateFormat sdf2 = new SimpleDateFormat( "yyyyMMdd-HH:mm:ss.SSS" );

	public DealServer( FIXSession session ) {
		this.session = session;
		session.addMessageReceiver( "D", this );
	}

	public void onMessage( MessageReceiveEvent ev ) {
		FIXMessage req = ev.getMessage();
		try {
			FIXMessage res = buildResponse( req );
			getSession().send( res );
		} catch ( Exception ex ) {
			logger.error( this, "response error", ex );
		}
	}

	FIXMessage buildResponse( FIXMessage req ) throws Exception {
		int dealId = getDealId();
		setDealId( dealId + 1 );
		FIXMessage res = getSession().getParser().createMessage( "8" );
		res.setField( "OrderID", "I" + df.format( dealId ));
		res.setField( "ClOrdID", req.getField( "ClOrdID" ));
		res.setField( "ExecID", "X" + df.format( dealId ));
		res.setField( "OrdStatus", "2" );
		res.setField( "Account", req.getField( "Account" ));
		res.setField( "Symbol", req.getField( "Symbol" ));
		res.setField( "ProductType", req.getField( "ProductType" ));
		res.setField( "Tenor", req.getField( "Tenor" ));
		res.setField( "TenorTo", req.getField( "TenorTo" ));
		res.setField( "Tenor2", req.getField( "Tenor2" ));
		String date;
		date = tenor.getDate( req.getField( "Tenor" ));
		res.setField( "SettlDate",
			date != null ? date : req.getField( "SettlDate" ));
		date = tenor.getDate( req.getField( "TenorTo" ));
		res.setField( "SettlDateTo",
			date != null ? date : req.getField( "SettlDateTo" ));
		date = tenor.getDate( req.getField( "Tenor2" ));
		res.setField( "SettlDate2",
			date != null ? date : req.getField( "SettlDate2" ));
		res.setField( "Side", req.getField( "Side" ));
		Date now = new Date();
		res.setField( "TradeDate", sdf1.format( now ));
		res.setField( "TransactTime", sdf2.format( now ));
		res.setField( "OrderQty", req.getField( "OrderQty" ));
		res.setField( "Price", req.getField( "Price" ));
		res.setField( "Currency", req.getField( "Currency" ));
		res.setField( "QuoteID", req.getField( "QuoteID" ));
		res.setField( "PartyRefNo", req.getField( "PartyRefNo" ));
		res.setField( "Text", "ID0001N: error message" );
		res.setField( "OrderQty2", req.getField( "OrderQty2" ));
		res.setField( "Price2", req.getField( "Price2" ));
		res.setField( "PartyRefNo2", req.getField( "PartyRefNo2" ));

		return res;
	}

	public void start() throws Exception {
		logger.info( this, "start server." );
	}

	public void stop() throws Exception {
		logger.info( this, "stop server." );
	}

	public synchronized void setDealId( int var ) {
		this.dealId = var;
	}
	public synchronized int getDealId() {
		return this.dealId;
	}

	public synchronized void setStop( boolean var ) {
		this.stop = var;
	}
	public synchronized boolean isStop() {
		return this.stop;
	}

	public synchronized void setSession( FIXSession var ) {
		this.session = var;
	}
	public synchronized FIXSession getSession() {
		return this.session;
	}

}
