package	jp.co.sds_corp.fixconnect.test;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import jp.co.sds_corp.fixconnect.FIXException;
import jp.co.sds_corp.fixconnect.transport.Acceptor;
import jp.co.sds_corp.fixconnect.transport.FIXSession;
import jp.co.sds_corp.fixconnect.transport.IStateChangeListener;
import jp.co.sds_corp.fixconnect.transport.ITransportEventListener;
import jp.co.sds_corp.fixconnect.transport.SessionState;
import jp.co.sds_corp.fixconnect.transport.StateChangeEvent;
import jp.co.sds_corp.fixconnect.transport.TransportEvent;
import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class DummyServer
	implements ITransportEventListener, IStateChangeListener
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	Acceptor acceptor;
	Thread	thrAcc;
	Map<String,SessionInfo> mapSession =
		new ConcurrentHashMap<String,SessionInfo>();
	static class SessionInfo
	{
		FIXSession	session;
		RateServer	rateServer;
		DealServer	dealServer;
		ReportServer	reportServer;
	}

	public void init( IConfiguration config ) throws Exception {
		acceptor = new Acceptor();
		acceptor.init( config );

		acceptor.addEventListener( this );
		acceptor.addStateChangeListener( this );
	}

	public void start() throws Exception {
		AcceptorThread acc = new AcceptorThread();
		thrAcc = new Thread( acc );
		thrAcc.setName( "aidw.fix.Acceptor" );
		thrAcc.start();
		for ( ;; ) {
			boolean bound = acceptor.isBound();
			if ( bound ) {
				logger.info( this, "server socket bounded." );
				break;
			}
			if ( !thrAcc.isAlive()) {
				logger.error( this, "accept thread not alive." );
				break;
			}
			Thread.sleep( 100 );
		}
	}

	class AcceptorThread implements Runnable
	{
		public void run() {
			try {
				acceptor.start();
			} catch ( Exception ex ) {
				logger.error( this, "acceptor error", ex );
			}
			logger.info( this, "accept thread terminated." );
		}
	}

	public void stop() throws Exception {
		acceptor.stop();
		thrAcc.join();
		for ( SessionInfo info : mapSession.values() ) {
			stopSession( info.session );
		}
	}

	public void onEvent( TransportEvent ev ) {
		// logger.debug( this, "event:" + ev, new Exception() );
		logger.debug( this, "event:" + ev );
/*
		FIXSession session = ev.getSession();
		switch ( ev.getType()) {
		case ACCEPT:
			startSession( session );
			break;
		}
*/
	}

	public void onException( FIXException ex ) {
		logger.debug( this, "exception:" + ex, ex );
	}

	public void stateChanged( StateChangeEvent ev ) {
		logger.debug( this, "state changed:" + ev );

		FIXSession session = ev.getSession();
		switch ( ev.getNewState() ) {
		// case LOGONED :
		case CONNECTED :
			if ( ev.getOldState() == SessionState.IDLE ) {
				startSession( session );
			} else
			if ( ev.getOldState() == SessionState.LOGONED ) {
				stopSession( session );
			}
			break;
		case IDLE:
			stopSession( session );
			break;
		}
	}

	void startSession( FIXSession session ) {
		logger.info( this, "start session " + session );
//		session.addEventListener( this );
		session.addStateChangeListener( this );

		SessionInfo info = new SessionInfo();
		info.session = session;
		info.rateServer = new RateServer( session );
		info.dealServer = new DealServer( session );
		info.reportServer = new ReportServer( session );
		mapSession.put( session.getSessionId(), info );
		try {
			info.rateServer.start();
			info.dealServer.start();
			info.reportServer.start();
		} catch ( Exception ex ) {
			logger.error( this, "start error", ex );
		}
		logger.info( this, "session started." + session );
	}

	void stopSession( FIXSession session ) {
		logger.info( this, "stop session " + session );
		SessionInfo info = mapSession.get( session.getSessionId() );
		if ( info == null ) {
			return;
		}
		try {
			info.rateServer.stop();
			info.dealServer.stop();
			info.reportServer.stop();
			info.session.close();
		} catch ( Exception ex ) {
			logger.error( this, "stop error", ex );
		}
		mapSession.remove( session.getSessionId() );
		logger.info( this, "session stopped." + session );
	}

	public static void main( String[] args ) {
		try {
			DummyServer server = new DummyServer();

			DOMConfiguration doc = DOMConfiguration.findResource( "test.xml" );
			server.init( doc.getChild("Server") );
			server.start();
		} catch ( Exception ex ) {
			logger.error( DummyServer.class, "server start error", ex );
		}
	}

}

