package	jp.co.sds_corp.fixconnect.test;

import java.text.DecimalFormat;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;

import jp.co.sds_corp.fixconnect.message.FIXMessage;
import jp.co.sds_corp.fixconnect.message.IMessageReceiver;
import jp.co.sds_corp.fixconnect.message.MessageReceiveEvent;
import jp.co.sds_corp.fixconnect.transport.FIXSession;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class ReportServer implements Runnable, IMessageReceiver
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	Thread	thread;
	boolean	stop;
	int	tradeId = 3;
	int	totalNum;
	List<FIXMessage> listReq = new Vector<FIXMessage>();
	Map<String,FIXMessage> mapReq =
		new ConcurrentHashMap<String,FIXMessage>();
	FIXSession session;
	Tenor	tenor = new Tenor();

	public ReportServer( FIXSession session ) {
		this.session = session;
		session.addMessageReceiver( "AD", this );
	}

	public void onMessage( MessageReceiveEvent ev ) {
		FIXMessage req = ev.getMessage();
		try {
			request( req );
		} catch ( Exception ex ) {
			logger.error( this, "response error", ex );
		}
	}

	void request( FIXMessage req ) throws Exception {
		String text = null;
		String type = req.getField( "SubscriptionRequestType" );
		if ( type == null ) {
			text = "SubscriptionRequestType not defined.";
			reject( req, text );
			return;
		}
		if ( type.equals( "0" )) {
			search( req, true );
		} else
		if ( type.equals( "1" )) {
			subscribe( req );
		} else
		if ( type.equals( "2" )) {
			unsubscribe( req );
		} else {
			text = "Unknwon SubscriptionRequestType[" + type + "]";
			reject( req, text );
		}
	}

	void reject( FIXMessage req, String text ) throws Exception {
		FIXMessage res = getSession().getParser().createMessage( "AQ" );
		String reqid = res.getField( "TradeRequestID" );
		res.setField( "TradeRequestID", reqid );
		res.setField( "TradeRequestResult", "99" );
		res.setField( "TradeRequestStatus", "2" );
		res.setField( "Text", text );
		getSession().send( res );
	}

	void search( FIXMessage req, boolean withAck ) throws Exception {
		String reqid = req.getField( "TradeRequestID" );
		logger.debug( this, "search [" + reqid + "]" );
		int num = 0;
		int max = getTradeId();
		for ( int i = 0; i < max; i++ ) {
			FIXMessage res = buildResponse( req, i + 1 );
			getSession().send( res );
			num++;
		}

		if ( withAck ) {
			FIXMessage ack = getSession().getParser().createMessage( "AQ" );
			ack.setField( "TradeRequestID", reqid );
			ack.setField( "TradeRequestType", req.getField( "TradeRequestType" ));
			ack.setField( "TotNumTradeRports", String.valueOf( num ));
			ack.setField( "TradeRequestResult", "0" );
			ack.setField( "TradeRequestStatus", "0" );
			ack.setField( "Text", "ID00001N:error message" );
			getSession().send( ack );
		}
	}

	void subscribe( FIXMessage req ) throws Exception {
		String text = null;
		String reqid = req.getField( "TradeRequestID" );
		logger.debug( this, "subscribe [" + reqid + "]" );
		if ( reqid == null ) {
			text = "TradeRequestID not defined.";
			reject( req, text );
			return;
		}

		FIXMessage org = mapReq.get( reqid );
		if ( org != null ) {
			text = "TradeRequestID[" + reqid + " duplicated.";
			reject( req, text );
			return;
		}

		setTotalNum( 0 );
		search( req, false );
		mapReq.put( reqid, req );
		listReq.add( req );
	}

	void unsubscribe( FIXMessage req ) throws Exception {
		String text = null;
		String reqid = req.getField( "TradeRequestID" );
		logger.debug( this, "unsubscribe [" + reqid + "]" );
		if ( reqid == null ) {
			text = "TradeRequestID not defined.";
			reject( req, text );
			return;
		}
		FIXMessage org = mapReq.remove( reqid );
		if ( org == null ) {
			text = "TradeRequestId[" + reqid + "] not found.";
			reject( req, text );
			return;
		}
		listReq.remove( org );

		FIXMessage res = getSession().getParser().createMessage( "AQ" );
		res.setField( "TradeRequestID", reqid );
		res.setField( "TradeRequestType", req.getField( "TradeRequestType" ));
		res.setField( "TotNumTradeRports", String.valueOf( getTotalNum() ));
		res.setField( "TradeRequestResult", "0" );
		res.setField( "TradeRequestStatus", "1" );
		res.setField( "Text", "ID00001N:error message" );
		getSession().send( res );
	}

	FIXMessage buildResponse( FIXMessage req, int tradeId ) throws Exception {
		DecimalFormat df = new DecimalFormat( "000000" );
		FIXMessage res;
		res = getSession().getParser().createMessage( "AE" );
		String reqid = req.getField( "TradeRequestID" );
		res.setField( "TradeRequestID", reqid );
		res.setField( "TradeID", df.format( tradeId ));
		res.setField( "SecondaryTradeID", "F00001" );
		res.setField( "ExecID", "X00001" );
		res.setField( "Symbol", "USD/JPY" );
		res.setField( "ClOrdID", "C00001" );
		res.setField( "Account", "A00001" );

		switch ( tradeId % 7 ) {
		case 0: buildSpot( res ); break;
		case 1: buildOutright( res ); break;
		case 2: buildSwap( res ); break;
		case 3: buildSO( res ); break;
		case 4: buildAG( res ); break;
		case 5: buildMod( res ); break;
		case 6: buildCan( res ); break;
		}
		res.setField( "Currency", "USD" );
		res.setField( "TradeDate", tenor.getDate( "ON" ));
		res.setField( "TransactTime", "20110221-01:23:45.678" );
		setTotalNum( getTotalNum() + 1 );
		res.setField( "PartyRefNo", "ABC123" );
		return res;
	}

	void buildSpot( FIXMessage res ) throws Exception {
		res.setField( "ExecSystem", "0" );
		res.setField( "ProductType", "SPOT" );
		res.setField( "SettlDate", tenor.getDate( "SP" ));
		res.setField( "Side", "1" );
		res.setField( "OrderQty", "10000.01" );
		res.setField( "Price", "82.34" );
	}

	void buildOutright( FIXMessage res ) throws Exception {
		res.setField( "ExecSystem", "0" );
		res.setField( "ProductType", "OR" );
		res.setField( "SettlDate", tenor.getDate( "SP" ));
		res.setField( "SettlDateTo", tenor.getDate( "1W" ));
		res.setField( "Side", "1" );
		res.setField( "OrderQty", "10000.01" );
		res.setField( "Price", "82.34" );
	}

	void buildSwap( FIXMessage res ) throws Exception {
		res.setField( "ExecSystem", "0" );
		res.setField( "ProductType", "SWAP" );
		res.setField( "SettlDate", tenor.getDate( "SP" ));
		res.setField( "SettlDate2", tenor.getDate( "1M" ));
		res.setField( "Side", "1" );
		res.setField( "OrderQty", "10000.01" );
		res.setField( "Price", "82.34" );
		res.setField( "OrderQty2", "10000.02" );
		res.setField( "Price2", "82.84" );
		res.setField( "PartyRefNo2", "DEF456" );
	}

	void buildSO( FIXMessage res ) throws Exception {
		res.setField( "ExecSystem", "L" );
		res.setField( "ProductType", "SO" );
		res.setField( "SettlDate", tenor.getDate( "SP" ));
		res.setField( "Side", "1" );
		res.setField( "OrderQty", "10000.01" );
		res.setField( "Price", "82.34" );
	}

	void buildAG( FIXMessage res ) throws Exception {
		res.setField( "ExecSystem", "L" );
		res.setField( "ProductType", "AG" );
		res.setField( "SettlDate", tenor.getDate( "SP" ));
		res.setField( "SettlDateTo", tenor.getDate( "1W" ));
		res.setField( "Side", "1" );
		res.setField( "OrderQty", "10000.01" );
		res.setField( "Price", "82.34" );
	}

	void buildMod( FIXMessage res ) throws Exception {
		res.setField( "OrigTradeID", "O00001" );
		res.setField( "OrigExecID", "O00002" );
		res.setField( "ExecSystem", "L" );
		res.setField( "ProductType", "MOD" );
		res.setField( "SettlDate", tenor.getDate( "SP" ));
		res.setField( "Side", "1" );
		res.setField( "OrderQty", "10000.01" );
		res.setField( "Price", "82.34" );
	}

	void buildCan( FIXMessage res ) throws Exception {
		res.setField( "OrigTradeID", "O00001" );
		res.setField( "OrigExecID", "O00002" );
		res.setField( "ExecSystem", "L" );
		res.setField( "ProductType", "CAN" );
		res.setField( "SettlDate", tenor.getDate( "SP" ));
		res.setField( "SettlDate2", tenor.getDate( "1M" ));
		res.setField( "Side", "1" );
		res.setField( "OrderQty", "10000.01" );
		res.setField( "Price", "82.34" );
		res.setField( "OrderQty2", "10000.02" );
		res.setField( "Price2", "82.84" );
	}

	public void start() throws Exception {
		logger.info( this, "start server." );
		thread = new Thread( this );
		thread.setName( "aidw.fix.ReportServer" );
		thread.start();
	}

	public void stop() throws Exception {
		logger.info( this, "stop server." );
		setStop( true );
		thread.join();
	}

	public void run() {
		for ( ;; ) {
			if ( isStop()) {
				break;
			}
			if ( getSession() == null ) {
				continue;
			}
			for ( FIXMessage req : listReq ) {
				try {
					int tradeId = incrementTradeId();

					FIXMessage res = buildResponse( req, tradeId );
					getSession().send( res );
				} catch ( Exception ex ) {
					logger.error( this, "send error", ex );
				}
			}
			try {
				Thread.sleep( 300 );
			} catch ( InterruptedException ex ) {
				logger.error( this, "interrupted", ex );
			}
		}
		logger.info( this, "server stopped." );
	}

	public synchronized void setTotalNum( int var ) {
		this.totalNum = var;
	}
	public synchronized int getTotalNum() {
		return this.totalNum;
	}

	public synchronized int incrementTradeId() {
		this.tradeId++;
		return this.tradeId;
	}
	public synchronized void setTradeId( int var ) {
		this.tradeId = var;
	}
	public synchronized int getTradeId() {
		return this.tradeId;
	}

	public synchronized void setStop( boolean var ) {
		this.stop = var;
	}
	public synchronized boolean isStop() {
		return this.stop;
	}

	public synchronized void setSession( FIXSession var ) {
		this.session = var;
	}
	public synchronized FIXSession getSession() {
		return this.session;
	}

}
