package jp.co.sds_corp.fixconnect.transport;

import jp.co.sds_corp.fixconnect.FIXException;
import jp.co.sds_corp.fixconnect.message.FIXMessage;
import jp.co.sds_corp.fixconnect.message.IMessageReceiver;
import jp.co.sds_corp.fixconnect.message.MessageReceiveEvent;
import jp.co.sds_corp.fixconnect.test.DummyServer;
import jp.co.sds_corp.test.BasicTestCase;
import jp.co.sds_corp.util.common.DumpTool;
import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
 * Unit test for simple App.
 */
public class InitiatorTest extends BasicTestCase
	implements IMessageReceiver, IStateChangeListener, ITransportEventListener
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	DumpTool dump = new DumpTool();
	DummyServer	server;
	Initiator	initiator;
	FIXMessage	message;

	@Override
	public void setUp() throws Exception {
		super.setUp();

		DOMConfiguration doc = DOMConfiguration.findResource( "test.xml" );

		IConfiguration config;
		config = doc.getChild( "Server" );
		setUpServer( config );

		config = doc.getChild( "Client" );
		setUpInitiator( config );
	}

	void setUpServer( IConfiguration config ) throws Exception {
		server = new DummyServer();
		server.init( config );
		server.start();
	}

	void setUpInitiator( IConfiguration config ) throws Exception {
		initiator = new Initiator();
		initiator.init( config );

		initiator.addMessageReceiver( this );
		initiator.addEventListener( this );
		initiator.addStateChangeListener( this );

		initiator.start();
    }

	public void onMessage( MessageReceiveEvent ev ) {
		FIXMessage msg = ev.getMessage();
		setReceivedMessage( msg );
		// logger.debug( this, "message: " + msg );
	}

	public synchronized void onEvent( TransportEvent ev ) {
		logger.debug( this, "event:" + ev );
	}
	public void onException( FIXException ex ) {
		logger.debug( this, "exception:" + ex, ex );
	}

	public void stateChanged( StateChangeEvent ev ) {
		logger.debug( this, "state changed:" + ev );
	}

	synchronized void setReceivedMessage( FIXMessage msg ) {
		this.message = msg;
	}
	synchronized FIXMessage getReceivedMessage() {
		return this.message;
	}

    /**
     */
/*
    public void testErr1() throws Exception {
		// logon();
		// requestQuote();
		order();
		// requestReport();
		// Thread.sleep( 5000 );
		// logout();
	}
*/

    /**
     */
    public void testSession() throws Exception {
		logon();
		requestQuote();
		order();
		requestReport();
		Thread.sleep( 2000 );
		logout();
	}

    /**
     */
    public void logon() throws Exception {
		initiator.logon();
	}

    /**
     */
    public void requestQuote() throws Exception {
		FIXMessage req;
		FIXMessage res;
		FIXSession session = initiator.getSession();

		req = session.getParser().createMessage( "R" );
		req.setField( "QuoteReqID", "00001" );
		req.setGroupCount( "Request", 1 );
		req.setField( "Request[0].Symbol", "USD/JPY" );
		req.setField( "Request[0].ProductType", "SPOT" );
		req.setField( "Request[0].Tenor", "SP" );
		session.send( req );

		session.send( req );

		req = session.getParser().createMessage( "R" );
		req.setField( "QuoteReqID", "00002" );
		req.setGroupCount( "Request", 1 );
		req.setField( "Request[0].Symbol", "USD/JPY" );
		req.setField( "Request[0].ProductType", "OR" );
		req.setField( "Request[0].Tenor", "1M" );
		req.setField( "Request[0].OrderQty", "10000.01" );
		session.send( req );

		req = session.getParser().createMessage( "R" );
		req.setField( "QuoteReqID", "00003" );
		req.setGroupCount( "Request", 1 );
		req.setField( "Request[0].Symbol", "USD/JPY" );
		req.setField( "Request[0].ProductType", "SWAP" );
		req.setField( "Request[0].Tenor", "TN" );
		req.setField( "Request[0].Tenor2", "1M" );
		req.setField( "Request[0].OrderQty", "10000.01" );
		req.setField( "Request[0].OrderQty2", "10000.02" );
		session.send( req );

		Thread.sleep( 3000 );

		req = session.getParser().createMessage( "Z" );
		req.setField( "QuoteReqID", "00001" );
		session.send( req );
		Thread.sleep( 500 );
	}

    /**
     */
    public void order() throws Exception {
		FIXMessage req;
		FIXMessage res;
		FIXSession session = initiator.getSession();

		req = session.getParser().createMessage( "D" );
		req.setField( "ClOrdID", "A0001" );
		session.send( req );
		Thread.sleep( 500 );
	}

    /**
     */
    public void requestReport() throws Exception {
		FIXMessage req;
		FIXMessage res;
		FIXSession session = initiator.getSession();

		req = session.getParser().createMessage( "AD" );
		req.setField( "TradeRequestID", "AD001" );
		req.setField( "TradeRequestType", "0" );
		req.setField( "SubscriptionRequestType", "0" );
		session.send( req );
		Thread.sleep( 500 );

		req = session.getParser().createMessage( "AD" );
		req.setField( "TradeRequestID", "AD002" );
		req.setField( "TradeRequestType", "0" );
		req.setField( "SubscriptionRequestType", "1" );
		session.send( req );

		session.send( req );

		Thread.sleep( 2000 );

		req = session.getParser().createMessage( "AD" );
		req.setField( "TradeRequestID", "AD002" );
		req.setField( "TradeRequestType", "0" );
		req.setField( "SubscriptionRequestType", "2" );
		session.send( req );
		Thread.sleep( 500 );
	}

    /**
     */
    public void logout() throws Exception {
		initiator.logout();
    }

	@Override
    public void tearDown() throws Exception {
		logger.debug( this, "tearDown" );
		initiator.stop();
		initiator.removeMessageReceiver( this );
		initiator.removeEventListener( this );
		initiator.removeStateChangeListener( this );

		logger.debug( this, "stop server" );
		server.stop();
		super.tearDown();
    }

}
